@@ -10,7 +10,7 @@ class TestSerialConsole(base.BaseNamespacedTestCase):
         super(TestSerialConsole, self).setUp()
         self.net = self.test_client.allocate_network(
             '192.168.242.0/24', True, True, '%s-net' % self.namespace)
-        self._await_network_ready(self.net['uuid'])
+        self._await_networks_ready([self.net['uuid']])
 
     def test_serial_console(self):
         inst = self.test_client.create_instance(
@@ -33,9 +33,5 @@ class TestSerialConsole(base.BaseNamespacedTestCase):
 
         self._await_login_prompt(inst['uuid'])
 
-        # We need to refresh our view of the instance, as it might have
-        # changed as it started up
-        inst = self.test_client.get_instance(inst['uuid'])
-
-        console = base.LoggingSocket(inst['node'], inst['console_port'])
+        console = base.LoggingSocket(self.test_client, inst)
         self.assertTrue(console.execute('uptime').find('load average'))
