@@ -10,29 +10,11 @@ from pydantic import (
 )
 
 
-def get_node_ip():
-    node_name = socket.getfqdn()
-    try:
-        return socket.gethostbyname(node_name)
-    except Exception:
-        # Only for localhost development environments
-        return '127.0.0.1'
-
-
 def get_node_name():
     return socket.getfqdn()
 
 
-class SFConfigBase(BaseSettings):
-    """
-    Separated from SFConfig for ease of testing
-    """
-
-    def get(self, key):
-        return self.dict()[key]
-
-
-class SFConfig(SFConfigBase):
+class SFConfig(BaseSettings):
     ###################
     # Deployment Wide #
     ###################
@@ -78,7 +60,7 @@ class SFConfig(SFConfigBase):
         16, description='How many vCPUS per real CPU'
     )
     RAM_OVERCOMMIT_RATIO: float = Field(
-        1.5,
+        3.0,
         description='How much vRAM per real unit of RAM, that is if Kernel '
                     'Shared Memory (KSM) is enabled, how much to overcommit '
                     'memory because of shared pages.'
@@ -94,8 +76,7 @@ class SFConfig(SFConfigBase):
                     'IPs'
     )
     NETWORK_NODE_IP: str = Field(
-        default_factory=get_node_ip,
-        description='IP of the node which will egress all traffic',
+        '', description='Mesh IP of the node which will egress all traffic',
     )
     DNS_SERVER: str = Field(
         '8.8.8.8',
@@ -178,10 +159,13 @@ class SFConfig(SFConfigBase):
         description='A URL listing all Ubuntu releases'
     )
 
+    MAX_IMAGE_TRANSFER_SECONDS: int = Field(
+        1800, description='How long to wait for an image transfer to occur before giving up'
+    )
+
     # Other options
-    GLUSTER_ENABLED: bool = Field(
-        False,
-        description='If we are using gluster for shared block storage.'
+    BLOB_REPLICATION_FACTOR: int = Field(
+        2, description='How many copies of each blob we like to have.'
     )
     ZONE: str = Field(
         'shakenfist', description='What nova called an availability zone'
@@ -191,29 +175,49 @@ class SFConfig(SFConfigBase):
     # Node Specific #
     #################
 
+    NODE_IS_ETCD_MASTER: bool = Field(
+        False, description='True if this node is an etcd master. This controls '
+                           'attempts to compact the master database.'
+    )
+    NODE_IS_HYPERVISOR: bool = Field(
+        False, description='True if this node is a hypervisor. This controls if '
+                           'VMs are started on this node or not.'
+    )
+    NODE_IS_NETWORK_NODE: bool = Field(
+        False, description='True if this node is the network node.'
+    )
+
     DISK_BUS: str = Field(
         'virtio',
         description='The bus to use for disk devices. One of virtio, scsi, '
                     'usb, ide, etc. See libvirt docs for full list of options.'
     )
-    DISK_FORMAT: str = Field(
-        'qcow',
-        description='The format for disks. Options are qcow (COW layers onto '
-                    'of image cache), qcow_flat (just qcow2, no COW), and flat '
-                    '(just raw disk).'
-    )
-    NODE_IP: str = Field(
-        default_factory=get_node_ip, description='IP of this node'
-    )
     NODE_NAME: str = Field(
         default_factory=get_node_name, description='FQDN of this node'
     )
+    NODE_EGRESS_IP: str = Field(
+        '', description='Egress IP of this node'
+    )
     NODE_EGRESS_NIC: str = Field(
-        'eth0', description='NIC for outbound traffic')
+        'eth0', description='NIC for outbound traffic'
+    )
+    NODE_MESH_IP: str = Field(
+        '', description='Mesh network IP of this node'
+    )
+    NODE_MESH_NIC: str = Field(
+        'eth0', description='NIC for virtual network mesh traffic'
+    )
     STORAGE_PATH: str = Field(
         '/srv/shakenfist', description='Where on disk instances are stored'
     )
 
+    LIBVIRT_USER: str = Field(
+        'libvirt-qemu', description='Name of the libvirt user'
+    )
+    LIBVIRT_GROUP: str = Field(
+        'libvirt-qemu', description='Name of the libvirt group'
+    )
+
     # Logging
     SLOW_LOCK_THRESHOLD: float = 5.0
     LOGLEVEL_API: str = 'info'
@@ -223,8 +227,8 @@ class SFConfig(SFConfigBase):
     LOGLEVEL_RESOURCES: str = 'info'
     LOGLEVEL_TRIGGERS: str = 'info'
     LOGLEVEL_QUEUES: str = 'info'
-    LOG_METHOD_TRACE: int = Field(
-        0, description='Add method name and module line number to log messages'
+    LOG_METHOD_TRACE: bool = Field(
+        False, description='Add method name and module line number to log messages'
     )
 
     class Config:
