@@ -6,6 +6,11 @@ class VersionSpecificationError(Exception):
     pass
 
 
+# Configuration
+class NoNetworkNode(Exception):
+    pass
+
+
 # Objects
 class ObjectException(Exception):
     pass
@@ -88,6 +93,10 @@ class BadCheckSum(Exception):
     pass
 
 
+class ImagesCannotShrinkException(Exception):
+    pass
+
+
 # Tasks
 class TaskException(Exception):
     pass
@@ -166,3 +175,11 @@ class ArtifactHasNoBlobs(ArtifactException):
 
 class BlobMissing(ArtifactException):
     pass
+
+
+class BlobRefCountDecrementBelowZero(ArtifactException):
+    pass
+
+
+class BlobDeleted(ArtifactException):
+    pass
