@@ -105,7 +105,7 @@ class SFCustomAdapter(logging.LoggerAdapter, PyLogrusBase):
             fields = {}
 
         # Handle "special fields" which might be internal objects
-        for key in ['instance', 'network', 'networkinterface', 'image']:
+        for key in ['artifact', 'blob', 'instance', 'network', 'networkinterface', 'node']:
             if key in fields:
                 value = fields[key]
                 if not isinstance(value, str):
@@ -130,7 +130,7 @@ class SFCustomAdapter(logging.LoggerAdapter, PyLogrusBase):
         msg = '%s[%s] %s' % (setproctitle.getproctitle(), os.getpid(), msg)
         kwargs["extra"] = self.extra
 
-        if config.get('LOG_METHOD_TRACE'):
+        if config.LOG_METHOD_TRACE:
             # Determine the name of the calling method
             filename = traceback.extract_stack()[-4].filename
             f_match = self.FILENAME_RE.match(filename)
