@@ -21,6 +21,7 @@ class FakeNetwork(object):
         self.dhcp_start = '10.0.0.2'
         self.netmask = '255.0.0.0'
         self.broadcast = '10.255.255.255'
+        self.provide_nat = True
 
 
 class FakeNetworkInterface(object):
@@ -219,11 +220,12 @@ class DHCPTestCase(testtools.TestCase):
     @mock.patch('shakenfist.dhcp.DHCP._send_signal', return_value=False)
     @mock.patch('shakenfist.dhcp.DHCP._make_config')
     @mock.patch('shakenfist.dhcp.DHCP._make_hosts')
-    @mock.patch('shakenfist.util.execute')
+    @mock.patch('shakenfist.util.process.execute')
     def test_restart_dhcpd(self, mock_execute, mock_hosts, mock_config,
                            mock_signal, mock_exists):
         d = dhcp.DHCP(FakeNetwork(), 'eth0')
         d.restart_dhcpd()
         mock_signal.assert_called_with(signal.SIGHUP)
         mock_execute.assert_called_with(
-            None, 'ip netns exec notauuid dnsmasq --conf-file=/a/b/c/dhcp/notauuid/config')
+            None, 'dnsmasq --conf-file=/a/b/c/dhcp/notauuid/config',
+            namespace='notauuid')
