@@ -1,6 +1,7 @@
 import base64
 import bcrypt
 import json
+import logging
 import mock
 
 
@@ -10,7 +11,7 @@ from shakenfist.baseobject import (
 from shakenfist.config import config, BaseSettings, SFConfig
 from shakenfist.external_api import app as external_api
 from shakenfist.ipmanager import IPManager
-from shakenfist.tests import test_shakenfist
+from shakenfist.tests import base
 
 
 class FakeResponse(object):
@@ -67,6 +68,7 @@ class FakeInstance(BaseFakeObject):
         self.power_state = {'power_state': power_state}
         self.placement = {'node': placement}
         self.version = 2
+        self.interfaces = []
 
 
 class FakeNetwork(BaseFakeObject):
@@ -81,6 +83,7 @@ class FakeNetwork(BaseFakeObject):
         self.name = name
         self.netblock = netblock
         self.version = 2
+        self.provide_nat = True
 
     def is_dead(self):
         return False
@@ -99,13 +102,24 @@ def _clean_traceback(resp):
     return resp
 
 
-class AuthTestCase(test_shakenfist.ShakenFistTestCase):
+class AuthTestCase(base.ShakenFistTestCase):
     def setUp(self):
         super(AuthTestCase, self).setUp()
 
         external_api.TESTING = True
         external_api.app.testing = True
         external_api.app.debug = False
+
+        external_api.app.logger.addHandler(logging.StreamHandler())
+        external_api.app.logger.setLevel(logging.DEBUG)
+        logging.root.setLevel(logging.DEBUG)
+
+        self.get_namespace = mock.patch('shakenfist.db.get_namespace')
+        self.mock_get_namespace = self.get_namespace.start()
+        self.addCleanup(self.get_namespace.stop)
+
+        # The client must be created after all the mocks, or the mocks are not
+        # correctly applied.
         self.client = external_api.app.test_client()
 
     def test_post_auth_no_args(self):
@@ -151,7 +165,7 @@ class AuthTestCase(test_shakenfist.ShakenFistTestCase):
             },
             resp.get_json())
 
-    @mock.patch('shakenfist.external_api.app.Auth._get_keys',
+    @mock.patch('shakenfist.external_api.auth.AuthEndpoint._get_keys',
                 return_value=(None, [_encode_key('cheese')]))
     def test_post_auth(self, mock_get_keys):
         resp = self.client.post(
@@ -159,7 +173,7 @@ class AuthTestCase(test_shakenfist.ShakenFistTestCase):
         self.assertEqual(200, resp.status_code)
         self.assertIn('access_token', resp.get_json())
 
-    @mock.patch('shakenfist.external_api.app.Auth._get_keys',
+    @mock.patch('shakenfist.external_api.auth.AuthEndpoint._get_keys',
                 return_value=('cheese', [_encode_key('bacon')]))
     def test_post_auth_not_authorized(self, mock_get_keys):
         resp = self.client.post(
@@ -172,20 +186,6 @@ class AuthTestCase(test_shakenfist.ShakenFistTestCase):
             },
             resp.get_json())
 
-    @mock.patch('shakenfist.etcd.get',
-                return_value={
-                    'service_key': 'cheese',
-                    'keys': {
-                        'key1': str(base64.b64encode(_encode_key('bacon')), 'utf-8'),
-                        'key2': str(base64.b64encode(_encode_key('sausage')), 'utf-8')
-                    }
-                })
-    def test_post_auth_service_key(self, mock_get):
-        resp = self.client.post(
-            '/auth', data=json.dumps({'namespace': 'banana', 'key': 'cheese'}))
-        self.assertEqual(200, resp.status_code)
-        self.assertIn('access_token', resp.get_json())
-
     def test_no_auth_header(self):
         resp = self.client.post('/auth/namespaces',
                                 data=json.dumps({
@@ -228,7 +228,38 @@ class AuthTestCase(test_shakenfist.ShakenFistTestCase):
         self.assertEqual(401, resp.status_code)
 
 
-class ExternalApiTestCase(test_shakenfist.ShakenFistTestCase):
+class AuthNoNamespaceMockTestCase(base.ShakenFistTestCase):
+    def setUp(self):
+        super(AuthNoNamespaceMockTestCase, self).setUp()
+
+        external_api.TESTING = True
+        external_api.app.testing = True
+        external_api.app.debug = False
+
+        external_api.app.logger.addHandler(logging.StreamHandler())
+        external_api.app.logger.setLevel(logging.DEBUG)
+        logging.root.setLevel(logging.DEBUG)
+
+        # The client must be created after all the mocks, or the mocks are not
+        # correctly applied.
+        self.client = external_api.app.test_client()
+
+    @mock.patch('shakenfist.etcd.get',
+                return_value={
+                    'service_key': 'cheese',
+                    'keys': {
+                        'key1': str(base64.b64encode(_encode_key('bacon')), 'utf-8'),
+                        'key2': str(base64.b64encode(_encode_key('sausage')), 'utf-8')
+                    }
+                })
+    def test_post_auth_service_key(self, mock_get):
+        resp = self.client.post(
+            '/auth', data=json.dumps({'namespace': 'banana', 'key': 'cheese'}))
+        self.assertEqual(200, resp.status_code)
+        self.assertIn('access_token', resp.get_json())
+
+
+class ExternalApiTestCase(base.ShakenFistTestCase):
     def setUp(self):
         super(ExternalApiTestCase, self).setUp()
 
@@ -245,31 +276,35 @@ class ExternalApiTestCase(test_shakenfist.ShakenFistTestCase):
         external_api.TESTING = True
         external_api.app.testing = True
         external_api.app.debug = False
-        self.client = external_api.app.test_client()
 
-        # Make a fake auth token
-        self.get_keys = mock.patch(
-            'shakenfist.external_api.app.Auth._get_keys',
-            return_value=('foo', ['bar'])
-        )
-        self.mock_get_keys = self.get_keys.start()
-        self.addCleanup(self.get_keys.stop)
-
-        resp = self.client.post(
-            '/auth', data=json.dumps({'namespace': 'system', 'key': 'foo'}))
-        print(resp.get_json())
-        self.assertEqual(200, resp.status_code)
-        self.auth_header = 'Bearer %s' % resp.get_json()['access_token']
+        external_api.app.logger.addHandler(logging.StreamHandler())
+        external_api.app.logger.setLevel(logging.DEBUG)
+        logging.root.setLevel(logging.DEBUG)
 
         fake_config = SFConfig(
             NODE_NAME='node1',
         )
 
-        self.config = mock.patch('shakenfist.instance.config',
-                                 fake_config)
+        self.config = mock.patch('shakenfist.instance.config', fake_config)
         self.mock_config = self.config.start()
         self.addCleanup(self.config.stop)
 
+        self.get_namespace = mock.patch('shakenfist.db.get_namespace')
+        self.mock_get_namespace = self.get_namespace.start()
+        self.addCleanup(self.get_namespace.stop)
+
+        # The client must be created after all the mocks, or the mocks are not
+        # correctly applied.
+        self.client = external_api.app.test_client()
+
+        # Make a fake auth token
+        with mock.patch('shakenfist.external_api.auth.AuthEndpoint._get_keys',
+                        return_value=('foo', ['bar'])):
+            resp = self.client.post(
+                '/auth', data=json.dumps({'namespace': 'system', 'key': 'foo'}))
+            self.assertEqual(200, resp.status_code)
+            self.auth_header = 'Bearer %s' % resp.get_json()['access_token']
+
 
 class ExternalApiGeneralTestCase(ExternalApiTestCase):
     def test_get_root(self):
@@ -339,24 +374,6 @@ class ExternalApiGeneralTestCase(ExternalApiTestCase):
             resp.get_json())
         self.assertEqual(403, resp.status_code)
 
-    @mock.patch('shakenfist.db.get_lock')
-    @mock.patch('shakenfist.etcd.get', return_value=None)
-    @mock.patch('shakenfist.etcd.put')
-    @mock.patch('bcrypt.hashpw', return_value='terminator'.encode('utf-8'))
-    def test_auth_add_key_new_namespace(self, mock_hashpw, mock_put, mock_get, mock_lock):
-        resp = self.client.post('/auth/namespaces',
-                                headers={'Authorization': self.auth_header},
-                                data=json.dumps({
-                                    'namespace': 'foo',
-                                    'key_name': 'bernard',
-                                    'key': 'cheese'
-                                }))
-        self.assertEqual(200, resp.status_code)
-        self.assertEqual('foo', resp.get_json())
-        mock_put.assert_called_with(
-            'namespace', None, 'foo',
-            {'name': 'foo', 'keys': {'bernard': 'dGVybWluYXRvcg=='}})
-
     @mock.patch('shakenfist.etcd.get_all',
                 return_value=[
                     ('/sf/namespace/aaa', {'name': 'aaa'}),
@@ -439,15 +456,6 @@ class ExternalApiGeneralTestCase(ExternalApiTestCase):
             },
             resp.get_json())
 
-    @mock.patch('shakenfist.db.get_lock')
-    @mock.patch('shakenfist.etcd.get', return_value={'keys': {'mykey': 'foo'}})
-    @mock.patch('shakenfist.etcd.put')
-    def test_delete_namespace_key(self, mock_put, mock_get, mock_lock):
-        resp = self.client.delete('/auth/namespaces/system/keys/mykey',
-                                  headers={'Authorization': self.auth_header})
-        self.assertEqual(200, resp.status_code)
-        mock_put.assert_called_with('namespace', None, 'system', {'keys': {}})
-
     @mock.patch('shakenfist.db.get_metadata', return_value={'a': 'a', 'b': 'b'})
     def test_get_namespace_metadata(self, mock_md_get):
         resp = self.client.get(
@@ -519,6 +527,8 @@ class ExternalApiGeneralTestCase(ExternalApiTestCase):
                          resp.get_json())
         self.assertEqual(404, resp.status_code)
 
+    # TODO(mikal): do better with covering interfaces here
+    @mock.patch('shakenfist.networkinterface.NetworkInterface.from_db')
     @mock.patch('shakenfist.instance.Instance._db_get',
                 return_value={
                     'cpus': 1,
@@ -536,7 +546,8 @@ class ExternalApiGeneralTestCase(ExternalApiTestCase):
                 })
     @mock.patch('shakenfist.instance.Instance._db_get_attribute',
                 return_value={})
-    def test_get_instance(self, mock_get_instance_attribute, mock_get_instance):
+    def test_get_instance(self, mock_get_instance_attribute, mock_get_instance,
+                          mock_get_interface):
         resp = self.client.get(
             '/instances/foo', headers={'Authorization': self.auth_header})
         self.assertEqual({
@@ -544,6 +555,7 @@ class ExternalApiGeneralTestCase(ExternalApiTestCase):
             'cpus': 1,
             'disk_spec': [{}],
             'error_message': None,
+            'interfaces': [],
             'memory': 2048,
             'name': 'barry',
             'namespace': 'namespace',
@@ -766,7 +778,7 @@ class ExternalApiInstanceTestCase(ExternalApiTestCase):
 
         self.addCleanup(self.config.stop)
 
-    @mock.patch('shakenfist.db.enqueue')
+    @mock.patch('shakenfist.etcd.enqueue')
     @mock.patch('shakenfist.instance.Instances',
                 return_value=[
                     FakeInstance(
@@ -891,10 +903,13 @@ class ExternalApiInstanceTestCase(ExternalApiTestCase):
     def test_post_instance_only_system_specifies_namespaces(
             self, mock_ipmanager, mock_lock, mock_net, mock_net_attribute,
             mock_get_artifact):
-        resp = self.client.post(
-            '/auth', data=json.dumps({'namespace': 'banana', 'key': 'foo'}))
-        self.assertEqual(200, resp.status_code)
-        non_system_auth_header = 'Bearer %s' % resp.get_json()['access_token']
+        with mock.patch('shakenfist.external_api.auth.AuthEndpoint._get_keys',
+                        return_value=('foo', ['bar'])):
+            resp = self.client.post(
+                '/auth', data=json.dumps({'namespace': 'banana', 'key': 'foo'}))
+            self.assertEqual(200, resp.status_code)
+            non_system_auth_header = 'Bearer %s' % resp.get_json()[
+                'access_token']
 
         resp = self.client.post('/instances',
                                 headers={
@@ -919,27 +934,57 @@ class ExternalApiInstanceTestCase(ExternalApiTestCase):
         self.assertEqual(401, resp.status_code)
 
 
-class ExternalApiNetworkTestCase(ExternalApiTestCase):
+class ExternalApiNetworkTestCase(base.ShakenFistTestCase):
     def setUp(self):
         super(ExternalApiNetworkTestCase, self).setUp()
 
-        class FakeConfig(BaseSettings):
-            NODE_NAME: str = 'seriously'
-            NODE_EGRESS_IP: str = '127.0.0.1'
-            NETWORK_NODE_IP = '127.0.0.1'
-            LOG_METHOD_TRACE: int = 1
-            NODE_EGRESS_NIC: str = 'eth0'
-            NODE_MESH_NIC: str = 'eth1'
+        self.add_event = mock.patch('shakenfist.db.add_event')
+        self.mock_add_event = self.add_event.start()
+        self.addCleanup(self.add_event.stop)
 
-        fake_config_network = FakeConfig()
-        self.config = mock.patch('shakenfist.config.config',
-                                 fake_config_network)
+        self.scheduler = mock.patch(
+            'shakenfist.scheduler.Scheduler', FakeScheduler)
+        self.mock_scheduler = self.scheduler.start()
+        self.addCleanup(self.scheduler.stop)
+
+        external_api.TESTING = True
+        external_api.app.testing = True
+        external_api.app.debug = False
+
+        external_api.app.logger.addHandler(logging.StreamHandler())
+        external_api.app.logger.setLevel(logging.DEBUG)
+        logging.root.setLevel(logging.DEBUG)
+
+        fake_config = SFConfig(
+            NODE_NAME='seriously',
+            NODE_EGRESS_IP='127.0.0.1',
+            NETWORK_NODE_IP='127.0.0.1',
+            LOG_METHOD_TRACE=1,
+            NODE_EGRESS_NIC='eth0',
+            NODE_MESH_NIC='eth1',
+            NODE_IS_NETWORK_NODE=True
+        )
+        self.config = mock.patch(
+            'shakenfist.external_api.base.config', fake_config)
         self.mock_config = self.config.start()
-        # Without this cleanup, other test classes will have
-        # 'shakenfist.config.config.get' mocked during parallel testing
-        # by stestr.
         self.addCleanup(self.config.stop)
 
+        self.get_namespace = mock.patch('shakenfist.db.get_namespace')
+        self.mock_get_namespace = self.get_namespace.start()
+        self.addCleanup(self.get_namespace.stop)
+
+        # The client must be created after all the mocks, or the mocks are not
+        # correctly applied.
+        self.client = external_api.app.test_client()
+
+        # Make a fake auth token
+        with mock.patch('shakenfist.external_api.auth.AuthEndpoint._get_keys',
+                        return_value=('foo', ['bar'])):
+            resp = self.client.post(
+                '/auth', data=json.dumps({'namespace': 'system', 'key': 'foo'}))
+            self.assertEqual(200, resp.status_code)
+            self.auth_header = 'Bearer %s' % resp.get_json()['access_token']
+
     @mock.patch('shakenfist.net.Network._db_get_attribute',
                 return_value={'value': dbo.STATE_CREATED, 'update_time': 2})
     @mock.patch('shakenfist.ipmanager.IPManager.from_db')
@@ -984,14 +1029,14 @@ class ExternalApiNetworkTestCase(ExternalApiTestCase):
                                  mock_ipmanager_from_db,
                                  mock_net_attribute):
 
+        self.client = external_api.app.test_client()
         resp = self.client.delete('/networks',
                                   headers={'Authorization': self.auth_header},
                                   data=json.dumps({
                                       'confirm': True,
                                       'namespace': 'foo'
                                   }))
-        self.assertEqual(['30f6da44-look-i-am-uuid'],
-                         resp.get_json())
+        self.assertEqual(['30f6da44-look-i-am-uuid'], resp.get_json())
         self.assertEqual(200, resp.status_code)
 
     @mock.patch('shakenfist.net.Network.from_db',
@@ -1026,3 +1071,76 @@ class ExternalApiNetworkTestCase(ExternalApiTestCase):
                                       'namespace': 'foo'
                                   }))
         self.assertEqual([], resp.get_json())
+
+
+class ExternalApiNoNamespaceMockTestCase(base.ShakenFistTestCase):
+    def setUp(self):
+        super(ExternalApiNoNamespaceMockTestCase, self).setUp()
+
+        self.add_event = mock.patch(
+            'shakenfist.db.add_event')
+        self.mock_add_event = self.add_event.start()
+        self.addCleanup(self.add_event.stop)
+
+        self.scheduler = mock.patch(
+            'shakenfist.scheduler.Scheduler', FakeScheduler)
+        self.mock_scheduler = self.scheduler.start()
+        self.addCleanup(self.scheduler.stop)
+
+        external_api.TESTING = True
+        external_api.app.testing = True
+        external_api.app.debug = False
+
+        external_api.app.logger.addHandler(logging.StreamHandler())
+        external_api.app.logger.setLevel(logging.DEBUG)
+        logging.root.setLevel(logging.DEBUG)
+
+        fake_config = SFConfig(
+            NODE_NAME='node1',
+        )
+
+        self.config = mock.patch('shakenfist.instance.config',
+                                 fake_config)
+        self.mock_config = self.config.start()
+        self.addCleanup(self.config.stop)
+
+        # The client must be created after all the mocks, or the mocks are not
+        # correctly applied.
+        self.client = external_api.app.test_client()
+
+        # Make a fake auth token
+        with mock.patch('shakenfist.external_api.auth.AuthEndpoint._get_keys',
+                        return_value=('foo', ['bar'])):
+            with mock.patch('shakenfist.db.get_namespace'):
+                resp = self.client.post(
+                    '/auth', data=json.dumps({'namespace': 'system', 'key': 'foo'}))
+                self.assertEqual(200, resp.status_code)
+                self.auth_header = 'Bearer %s' % resp.get_json()[
+                    'access_token']
+
+    @mock.patch('shakenfist.db.get_lock')
+    @mock.patch('shakenfist.etcd.get', return_value={'keys': {'mykey': 'foo'}})
+    @mock.patch('shakenfist.etcd.put')
+    def test_delete_namespace_key(self, mock_put, mock_get, mock_lock):
+        resp = self.client.delete('/auth/namespaces/system/keys/mykey',
+                                  headers={'Authorization': self.auth_header})
+        self.assertEqual(200, resp.status_code)
+        mock_put.assert_called_with('namespace', None, 'system', {'keys': {}})
+
+    @mock.patch('shakenfist.db.get_lock')
+    @mock.patch('shakenfist.etcd.get', return_value=None)
+    @mock.patch('shakenfist.etcd.put')
+    @mock.patch('bcrypt.hashpw', return_value='terminator'.encode('utf-8'))
+    def test_auth_add_key_new_namespace(self, mock_hashpw, mock_put, mock_get, mock_lock):
+        resp = self.client.post('/auth/namespaces',
+                                headers={'Authorization': self.auth_header},
+                                data=json.dumps({
+                                    'namespace': 'foo',
+                                    'key_name': 'bernard',
+                                    'key': 'cheese'
+                                }))
+        self.assertEqual(200, resp.status_code)
+        self.assertEqual('foo', resp.get_json())
+        mock_put.assert_called_with(
+            'namespace', None, 'foo',
+            {'name': 'foo', 'keys': {'bernard': 'dGVybWluYXRvcg=='}})
